﻿using System;
using System.Collections.Generic;
using System.Data.SQLite;
using MtSql.SqLite;

namespace MtSql.ConsoleDemo
{
    class Program
    {
        static string connectionString = "Data Source=Database.db";

        static void Main(string[] args)
        {
            //Create the database wrapper
            var db = new MtSqLiteDb(connectionString);

            InsertData(db);

            ReadData(db);
            Console.ReadKey();
        }

        /// <summary>
        /// Insert some Data into the article table
        /// </summary>
        static void InsertData(MtSqLiteDb db)
        {
            var name = "TestItem";
            ArticleWeightClass c = ArticleWeightClass.High;

            db.ExecuteSqlCommand("INSERT INTO Article(name,weightclass) VALUES(@name,@weightclass)",
                new List<SQLiteParameter>().AddParameter("@name", name).AddParameter("@weightclass", (long)c));
        }

        /// <summary>
        /// Reading all rows from article table
        /// </summary>
        static void ReadData(MtSqLiteDb db)
        {
            db.ExecuteReader("SELECT id,name FROM Article", (cols, rw) =>
            {
                var id = rw.Get<long>("id", cols);
                var name = rw.Get<string>("name", cols);
                var wclass = rw.Get<ArticleWeightClass>("weightclass", cols);
                Console.WriteLine(string.Format("ID: {0} | Name: {1} | Weight Class: {2}", id, name, wclass));
                return true; //Continue reading the next row, returning false will stop reading
            });
        }
    }

    /// <summary>
    /// A simple example enum, for demonstration purposes.
    /// To be compatible with SqLite, it derives from long instead of int
    /// </summary>
    public enum ArticleWeightClass : long
    {
        High = 1,
        Medium = 2,
        Low = 3
    }
}
