﻿using System.Collections.Generic;
using System.Data;
using System.Data.Odbc;
/**
 * Jan Lorenz, 30.03.2011
 **/
namespace MtSql.Odbc
{
    public static class OdbcExtensions
    {
        /// <summary>
        /// Chainable, Adds a new OdbcParameter to the List
        /// </summary>
        public static List<OdbcParameter> AddParameter(this List<OdbcParameter> ps, object value)
        {
            ps.Add(new OdbcParameter(string.Empty, value));
            return ps;
        }

        /// <summary>
        /// Chainable, Adds a new OdbcParameter to the List
        /// </summary>
        public static List<OdbcParameter> AddParameter(this List<OdbcParameter> ps, string name, object value)
        {
            ps.Add(new OdbcParameter(name, value));
            return ps;
        }

        /// <summary>
        /// Chainable, Adds a new OdbcParameter to the List
        /// </summary>
        public static List<OdbcParameter> AddParameter(this List<OdbcParameter> ps, string name, OdbcType type, object value)
        {
            OdbcParameter p = new OdbcParameter(name, type);
            p.Value = value;
            ps.Add(p);
            return ps;
        }

        /// <summary>
        /// Chainable, Adds a new OdbcParameter to the List
        /// </summary>
        public static List<OdbcParameter> AddParameter(this List<OdbcParameter> ps, string name, OdbcType type, int size, object value)
        {
            OdbcParameter p = new OdbcParameter(name, type, size);
            p.Value = value;
            ps.Add(p);
            return ps;
        }

        /// <summary>
        /// Chainable, Adds a new Odbc-OutParameter to the List
        /// </summary>
        public static List<OdbcParameter> AddOutParameter(this List<OdbcParameter> ps, string name, object value, out OdbcParameter outParam)
        {
            outParam = new OdbcParameter(name, value);
            outParam.Direction = ParameterDirection.Output;
            ps.Add(outParam);
            return ps;
        }

        /// <summary>
        /// Chainable, Adds a new Odbc-OutParameter to the List
        /// </summary>
        public static List<OdbcParameter> AddOutParameter(this List<OdbcParameter> ps, string name, OdbcType type, object value, out OdbcParameter outParam)
        {
            outParam = new OdbcParameter(name, type);
            outParam.Direction = ParameterDirection.Output;
            outParam.Value = value;
            ps.Add(outParam);
            return ps;
        }

        /// <summary>
        /// Chainable, Adds a new Odbc-OutParameter to the List
        /// </summary>
        public static List<OdbcParameter> AddOutParameter(this List<OdbcParameter> ps, string name, OdbcType type, int size, object value, out OdbcParameter outParam)
        {
            outParam = new OdbcParameter(name, type, size);
            outParam.Direction = ParameterDirection.Output;
            outParam.Value = value;
            ps.Add(outParam);
            return ps;
        }
    }
}
