﻿using System;
using System.Collections.Generic;
using System.Data;

namespace MtSql
{
    public abstract class DataSourceBase<TCommand, TParameter> : IDataSource<TParameter>
         where TCommand : IDbCommand
         where TParameter : IDataParameter
    {
        #region Abstract Methods

        protected abstract TCommand GetCommand(string sqlCommand, CommandType type, List<TParameter> parameters);

        protected abstract void CloseCommand(IDbCommand cmd);

        #endregion

        #region Execute Command

        public int ExecuteSqlCommand(string command)
        {
            return ExecuteSqlCommand(command, CommandType.Text, new List<TParameter>());
        }

        public int ExecuteSqlCommand(string sqlCommand, List<TParameter> parameters)
        {
            return ExecuteSqlCommand(sqlCommand, CommandType.Text, parameters);
        }

        public int ExecuteSqlCommand(string sqlCommand, CommandType type, List<TParameter> parameters)
        {
            var cmd = GetCommand(sqlCommand, type, parameters);
            int retval = 0;
            try { retval = cmd.ExecuteNonQuery(); }
            finally { CloseCommand(cmd); }
            return retval;
        }

        #endregion

        #region DataReader

        public void ExecuteReader(string sqlCommand, RowCallback callback)
        {
            ExecuteReader(sqlCommand, CommandType.Text, callback);
        }

        public void ExecuteReader(string sqlCommand, CommandType type, RowCallback callback)
        {
            ExecuteReader(sqlCommand, type, new List<TParameter>(), callback);
        }

        public void ExecuteReader(string sqlCommand, CommandType type, List<TParameter> parameters, RowCallback callback)
        {
            var cmd = GetCommand(sqlCommand, type, parameters);
            try
            {
                using (var rd = cmd.ExecuteReader())
                {
                    int fc = rd.FieldCount;
                    var columns = new LockableDictionary<string, int>();
                    for (int i = 0; i < rd.FieldCount; ++i)
                        columns.Add(rd.GetName(i), i);
                    object[] row = new object[fc];
                    columns.Lock();
                    while (rd.Read())
                    {
                        rd.GetValues(row);
                        if (!callback(columns, row))
                            break;
                    }
                }
            }
            finally { CloseCommand(cmd); }
        }

        #endregion
    }
}
