﻿using System;
/**
 * Jan Lorenz, 30.03.2011
 **/
namespace MtSql
{
    public static class ExtensionMethods
    {
        #region Conversion Methods

        public static object ValueOrDbNull<T>(this Nullable<T> value) where T : struct
        {
            if (value.HasValue) return value.Value;
            return DBNull.Value;
        }

        public static object ValueOrDbNull<T>(this T value) where T : class
        {
            if (value != null)
                return value;
            return DBNull.Value;
        }

        public static T Get<T>(this object[] row, string columnName, LockableDictionary<string, int> columns)
        {
            if (row != null && columns.ContainsKey(columnName))
                return Get<T>(row[columns[columnName]]);
            return default(T);
        }

        public static T Get<T>(object value)
        {
            var tType = typeof(T);
            var rType = value.GetType();
            if (tType.IsEnum)
            {
                if (Enum.IsDefined(tType, value))
                {
                    if (rType.IsPrimitive)
                        return (T)value;
                    else
                        return (T)Enum.Parse(tType, value.ToString());
                }
            }
            else if (tType == rType || (tType.IsGenericType && Nullable.GetUnderlyingType(tType) == rType))
                return (T)value;
            return default(T);
        }

        #endregion
    }
}
