﻿using System;
using System.Collections.Generic;

namespace MtSql
{
    public class LockableDictionary<TKey, TValue> : Dictionary<TKey, TValue>
    {
        private bool _IsLocked = false;

        public void Lock()
        {
            _IsLocked = true;
        }

        public new void Add(TKey key, TValue value)
        {
            if (_IsLocked)
                throw new InvalidOperationException("Dictionary is locked");
            base.Add(key, value);
        }

        public new TValue this[TKey key]
        {
            get
            {
                return base[key];
            }
            set
            {
                if (_IsLocked)
                    throw new InvalidOperationException("Dictionary is locked");
                base[key] = value;
            }
        }

        public new void Remove(TKey key)
        {
            if (_IsLocked)
                throw new InvalidOperationException("Dictionary is locked");
            base.Remove(key);
        }
    }
}
