﻿using System;
using System.Collections.Generic;
using System.Data;

namespace MtSql
{
    public abstract class MtDb<TConnection, TTransaction, TCommand, TParameter> : DataSourceBase<TCommand, TParameter>
        where TParameter : IDataParameter
        where TConnection : IDbConnection
        where TCommand : IDbCommand
        where TTransaction : IDbTransaction
    {
        private readonly Func<TConnection> _connectionCreator;

        #region Ctor

        public MtDb(Func<TConnection> connectionCreator)
        {
            _connectionCreator = connectionCreator;
        }

        #endregion

        #region Support Methods

        protected override TCommand GetCommand(string sqlCommand, CommandType type, List<TParameter> parameters)
        {
            var c = _connectionCreator();
            c.Open();
            var cmd = c.CreateCommand();
            foreach (var p in parameters)
                cmd.Parameters.Add(p);
            cmd.CommandText = sqlCommand;
            cmd.CommandType = type;
            return (TCommand)cmd;
        }

        protected override void CloseCommand(IDbCommand cmd)
        {
            cmd.Connection.Close();
            cmd.Dispose();
        }

        #endregion

        #region Transactions

        public MtTransaction<TConnection, TTransaction, TCommand, TParameter> GetTransaction(IsolationLevel level)
        {
            var con = _connectionCreator();
            return new MtTransaction<TConnection, TTransaction, TCommand, TParameter>(con, level, (c, t) =>
            {
                TCommand cmd = (TCommand)c.CreateCommand();
                cmd.Connection = c;
                cmd.Transaction = t;
                return cmd;
            });
        }

        public void Commit(MtTransaction<TConnection, TTransaction, TCommand, TParameter> transaction)
        {
            try { transaction.Transaction.Commit(); }
            finally { transaction.Connection.Close(); transaction.Connection.Dispose(); }
        }

        public void Rollback(MtTransaction<TConnection, TTransaction, TCommand, TParameter> transaction)
        {
            try { transaction.Transaction.Rollback(); }
            finally { transaction.Connection.Close(); transaction.Connection.Dispose(); }
        }

        #endregion
    }
}
